<?php
// $Id: default.php,v 1.12 2007/05/16 01:24:42 agentken Exp $

/**
 * @file
 * The default page layout file for MySite.  Required.
 */

/**
 * The theme function for this layout.
 *
 * @param $content
 * An array of content to display.  This content consists of up to 4 sub-array elements:
 *    'owner' == the $user object of the owner of this MySIte page.
 *    'mysite' == information about this MySite page.
 *    'data' == the content data to display.  This data will be formatted by the selected format file.
 *    'header' == used to display any messages waiting for the user.
 *
 * @ingroup plugins
 * @ingroup themes
 */
function theme_mysite_default_layout($content) {
  // break the array into pieces
  $owner = $content['owner'];
  $mysite = $content['mysite'];
  $columns = mysite_layout_default();
  $data = mysite_prepare_columns($mysite, $content['data'], $columns['count']);
  $header = $content['header'];
  // print the header message, if present
  if (isset($header)) {
    $output =  '<div class="messages">'.  $header .'</div>';
  }
  // wrap the output
  $output .= '<div id="mysite">';
  // set up the drag-and-drop styles
  $output .= '<blockquote class="mysite-sortable" id="mysite-sort0">';  
  // cycle through the data
  foreach ($data as $key => $value) {
    if ($value['mid']) {
      $output .= '<div class="group sortable-item" id="'. $value['mid'] .'">';
    }
    else {
      $output .= '<div class="group">';
    }
    $output .= '<fieldset class="collapsible mysite-bar">';
    $output .= '<legend>'. $value['title'] .'</legend>';
    if (!empty($value['output'])) {
      if (!empty($value['output']['image'])) {
        $output .= $value['output']['image'];
      } 
      $output .= theme('mysite_'. $value['format'] .'_item', $value['output']['items']);
    }
    else {
      $output .= t('<p>No content found.</p>');
    }
    $output .= ' <span class="actions">'. $value['actions'] .'</span> ';
    if (!empty($value['output']['base'])) {
      $output .= '<span class="more">'. l(t('Read more'), $value['output']['base']) .'</span>';
    }
    $output .= '</fieldset>';
    $output .= '</div>';    
  }
  $output .= '</blockquote>';
  $output .= '</div>';
  print theme('page', $output, variable_get('mysite_fullscreen', 1));
  return;
}

/**
 * Information about this layout
 */
function mysite_layout_default() {
  $data = array();
  $data['regions'] = array('0' => t('Main'));
  $data['count'] = count($data['regions']);
  $data['name'] = t('Basic layout');
  $data['description'] = t('A single-column display of your personal content.');
  $data['image'] = 'default.png';  
  return $data;
}