<?php
// $Id: node.inc,v 1.5 2007/05/16 01:24:42 agentken Exp $

/**
 * These files are included as needed to reduce overhead.
 * See the README for additional information.
 */

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_node($get_options = TRUE) {
  if (module_exists('node')) {
    $type = array(
      'name' => t('Posts'),
      'description' => t('<b>Posts</b>: Content posts of a specific type.'),
      'include' => 'node',
      'prefix' => t(''),
      'suffix' => t('posts'),
      'category' => t('Content'),
      'weight' => 0,
      'form' => FALSE,
      'label' => t('Add Posts'),
      'help' => t('You can add recent posts by type to your personal page. Choose from the list of available content types.'),
      'search' => FALSE
    );
    if ($get_options) {
      $type['options'] = mysite_type_node_options();
    }
    return $type;
  }
}

/**
 * Check to see if this include can be enabled
 */
function mysite_type_node_active($type) {
  // some users must be allowed to view content, otherwise, give a configuration message
  $result = db_query("SELECT perm FROM {permission}");
  $check = '';
  $br = '';
  $message = '';
  while ($perms = db_fetch_object($result)) {
    $check .= $perms->perm;
  }
  if (stristr($check, 'access content')) {
    $value = TRUE;
  }
  else {
    $value = FALSE;
    $message = l(t('No users have permission to access site content'), 'admin/user/access');
    $br = '<br />';
  }
  // there must be at least one allowed type
  $allowed = variable_get('mysite_node_settings', array());
  // if this variable is saved as blank, then a 0 => 1 value is present
  unset($allowed[0]);
  if (empty($allowed)) {
    $value = FALSE;
    $message = $br . l(t('At least one content type must be configured for MySite.'), 'admin/settings/mysite/type/node');
  }
  return array($type => $value, 'message' => $message);
}

/**
 * Set the options for this type
 */
function mysite_type_node_options() {
  $options = array();
  // we have to get the allowed options and compare them to the stored options
  $default = variable_get('mysite_node_settings', array());
  // if this variable is saved as blank, then a 0 => 1 value is present
  unset($default[0]);
  $content = array_flip($default);
  unset($content[0]);
  $types = mysite_get_custom('node', NULL, TRUE);
  if (!empty($content) && !empty($types)) {
    foreach ($types as $type) {
      if (in_array($type->type_key, $content)) {
        $options['name'][] = mysite_type_node_title($type->myid);
        $options['type_id'][] = $type->myid;
        $options['type'][] = 'node';
        $options['icon'][] = mysite_get_icon('node', $type->myid);
      }
    }
  }
  return $options;
}

/**
 * Return the title if given an id
 */
function mysite_type_node_title($type_id = NULL, $title = NULL) {
  if (!empty($type_id)) {
    if (is_null($title)) {
      $data = mysite_get_custom('node', $type_id, FALSE);
      $title = node_get_types('name', $data->type_key);
    }
    $type = mysite_type_node(FALSE);
    $title = $type['prefix'] .' '. $title .' '. $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_node_data($type_id = NULL, $settings = NULL) {
  if (!empty($type_id)) {
    $data = mysite_get_custom('node', $type_id, FALSE);
    $sql = db_rewrite_sql("SELECT n.nid, n.changed FROM {node} n WHERE n.type = '%s' AND n.status = 1 ORDER BY n.changed DESC");
    $result = db_query_range($sql, $data->type_key, 0, variable_get('mysite_elements', 5));

    $data = array(
      'base' => '',
      'xml' => ''
      );
    $items = array();
    $i = 0;
    while ($nid = db_fetch_object($result)) {
      $node = node_load($nid->nid);
      $items[$i]['type'] = $node->type;
      $items[$i]['link'] = l($node->title, 'node/'. $nid->nid);
      $items[$i]['title'] = check_plain($node->title);
      $items[$i]['subtitle'] = NULL; 
      $items[$i]['date'] = $node->changed;
      $items[$i]['uid'] = $node->uid;
      $items[$i]['author'] = check_plain($node->name);
      $items[$i]['teaser'] = mysite_teaser($node);
      $items[$i]['nid'] = $node->nid;
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;
}

/**
 * Return block settings for this type
 *
 * @param $type is an additional parameter here, since this is a special case
 */
function mysite_type_node_block_node($nid = NULL, $type = NULL) {
  global $user;
  $types = variable_get('mysite_node_settings', array());
  if (in_array($type, $types)) {
    $info = mysite_get_myid('node', $type);
    if (!empty($info)) {
      $data = array();
      $data['uid'] = $user->uid;
      $data['type'] = 'node';
      $data['type_id'] = $info;
      $data['title'] = mysite_type_node_title($info);
      $content = mysite_block_handler($data);
      return $content;
    }
  }
}

/**
 * Cron cleanup function
 */
function mysite_type_node_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  $sql = "SELECT mid, uid, type_id, title FROM {mysite_data} WHERE type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT myid FROM {mysite_content} WHERE type = 'node' AND myid = %d";
    $check = db_fetch_object(db_query($sql, $item['type_id']));
    if (empty($check->myid)) {
      $data[$item['mid']] = $item;
    }
  }
  return $data;
}

/**
 * Settings for this type
 */
function mysite_type_node_settings() {
  mysite_check_settings('node');
  // when this page is configured, we need to establish the data in the {mysite_content} table.
  // note that we do not delete existing entries in case they are ever enabled again.
  $default = variable_get('mysite_node_settings', array());
  // if this variable is saved as blank, then a 0 => 1 value is present
  unset($default[0]);
  // get the content types
  $types = node_get_types();
  ksort($types);
  $options = array();
  foreach ($types as $key => $value) {
    $options[$key] = "<b>$value->name</b>: ". $value->description;
  }
  // check to see which types have been created
  if (!empty($default)) {
    foreach ($default as $key => $value) {
      if (!empty($value)) {
        $id = mysite_get_myid('node', $key);
        if (is_numeric($id) && !in_array($key, $default)) {
          $inactive[$id] = $options[$key];
        }
      }
    }
  }
  // check to see if we have any dead types in the database
  $inactive = array();
  $result = db_query("SELECT myid, type_key FROM {mysite_content} WHERE type = 'node'");
  while ($data = db_fetch_object($result)) {
    if (empty($default[$data->type_key])) {
      $inactive[$data->myid] = $options[$data->type_key];
    } 
  }
  $output = drupal_get_form('mysite_type_node_settings_form', $default, $types, $options);
  if (!empty($inactive)) {
    $output .= drupal_get_form('mysite_type_node_delete_form', $default, $types, $inactive);
  }
  return $output;
}

/**
 * FormsAPI for mysite_type_node_settings
 */
function mysite_type_node_settings_form($default, $types, $options) {
  $form['mysite_node_settings'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Allowed content types'),
    '#prefix' => t('<h3>Content types</h3><p>Enabling the options below will allow users to add recent posts of a specific content type to their MySite page.</p>'),
    '#default_value' => $default,
    '#options' => $options,
    '#required' => FALSE,
    '#description' => t('Select the content types that users may select for their MySite page.')
  ); 
  return system_settings_form($form);     
}


/**
 * FormsAPI for deleting an inactive node type from {mysite_content}
 */
function mysite_type_node_delete_form($default, $types, $inactive) {
  $form['mysite_nodetype_delete'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Inactive content types'),
    '#prefix' => t('<h3>Inactive content types</h3><p>The following content types have been created and disabled.  If you wish to delete them, select from the list below. <em>You will not be asked to confirm this deletion.</em></p>'),
    '#default_value' => $default,
    '#options' => $inactive,
    '#required' => FALSE,
    '#description' => t('Delete inactive content types from MySite.')
  ); 
  $form['mysite_nodetype_confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm deletion'),
    '#return_value' => 1,
    '#default_value' => 0,
    '#description' => t('You must confirm that you wish to delete the selected items.')
  );
  $form['mysite_node_delete_submit'] = array('#type' => 'submit', '#value' => t('Delete selections'));
  return $form;
}

/**
 * FormsAPI for deleting inactive node types
 */
function mysite_type_node_delete_form_submit($form_id, $form_values) {
  $reset = 0;
  foreach ($form_values['mysite_nodetype_delete'] as $key => $value) {
    if (!empty($value) && $key == $value) {
      if ($form_values['mysite_nodetype_confirm'] == 1) {
        db_query("DELETE FROM {mysite_content} WHERE myid = %d", $value);
        $reset = 1;
      }
    }
  }
  if ($reset) {
    variable_set('mysite_cron', 0);
    drupal_set_message(t('Item deleted. <a href="!url">Reset all user MySites now?</a>.  If you choose not to reset now, user pages will be updated at the next cron run.', array('!url' => 'admin/settings/mysite')));
  }
  else {
    drupal_set_message(t('Deletion failed.  Did you confirm the items you wish to delete?'));
  }
}
